module Gill

  class Parse

    attr_accessor :category, :folder, :repo

    # Initialize a ne wparse object.
    #
    # @param [String] uri
    #   The string to be parsed and retunred to create a gill cloned repository.
    #
    def initialize(uri)
      @uri = uri.split(/\#/)
      self.start
    end

    #
    # Setter for the parsed objects category.
    #
    def category=(category)
      @category = category
    end

    #
    # Setter for the parsed objects folder.
    #
    def folder=(folder)
      @folder = folder
    end

    #
    # Setter for the parsed objects repository uri.
    #
    # @param [String] repo
    #   The parsed objects repository uri.
    #
    # @return [String] repo
    #   The uri for the repository.
    # 
    def repo=(repo)
      @repo = repo
    end

    #
    # Set and verefiy the category, folder and repository uri for the parsed object.
    #
    def start
      self.category = @uri.first
      
      if @uri.length == 3
        
        self.repo = @uri.first if valid?(@uri.first)
        self.category = @uri[1]
        self.folder = @uri.last      
        
      elsif @uri.length == 2
        
        self.category = @uri.last
        self.repo = @uri.first if valid?(@uri.first)
        self.folder = get_folder(self.repo)
        
      elsif @uri.length < 2
        
        self.category = false
        self.repo = @uri.first if valid?(@uri.first)
        self.folder = get_folder(self.repo)
        
      else
        
        raise(ArgumentError, "too many arguments.")
        
      end
    end
    
    #
    # Find the folder name from the repository uri.
    #
    # @param [String] repo
    #   The uri for the remote repository.
    # 
    # @return [String] repo
    #   The folder/project name of the passed uri.
    # 
    def get_folder(repo)
      
      if repo =~ /\.git/i
        repo_name = repo.match(/\S+[\/|\:](\S+)\.git/i)[1]
      else
        repo_name = repo.match(/\S+[\/|\:](\S+)\Z/i)[1]
      end
      
      repo_name
    end

    #
    # Check to verify the passed uri is a valid repository uri.
    #
    # @param [String] repo
    #   The uri to the remote repository.
    # 
    # @return [String] repo
    #   Return the repository uri if it is indeed a valid uri.
    # 
    def valid?(repo)
      case repo
        when repo[/\A(http|https|git)\:\/\/\S+[\:|\/]\S+[\.git|\Z]\Z/i]
          return repo
        when repo[/http\:\/\/\S+\Z|https\:\/\/\S+\Z|git\:\/\/\S+\Z/i]
          return repo
        when repo[/\A(\S+\@\b\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\b\:\S+\S+[\.git|\Z])\Z/i]
          return repo
        when repo[/\A\S+\@\S+[\:|\/]\S+[\.git|\Z]\Z/i]
          return repo
        else
          raise(RepositoryError, 'not a git repository.')
      end
    end

  end
end
