module Gill

  class Remove
    include Helper
    include Settings

    attr_reader :category, :category_path, :repo_name, :repo_path

    #
    # Initialize New Remove Object
    # 
    # @param [String] args Arguments - Path,RepoName,Category
    #
    def initialize(args)
      @args = args.split(/\#/)
      @category, @repo_name = @args.first, @args.last
      @category_path = File.join(Gill.config.source_path, @category)
      @repo_path = File.join(@category_path, @repo_name)
      @parent_folders = []
    end

    #
    # Remove Repo
    # 
    # @param [true,false] force Forcefully remove a repository.
    #
    def remove_repo(force=false)
      remove_repository_files if category_exists? && repo_exists?
    end

    private

      def clean_cache
        Gill.config.clean_cache
      end

      def cached_repo_path
        return Gill.config.categories[@category.to_sym][@repo_name.to_sym][:path] if Gill.config.categories.has_key?(@category.to_sym)
        raise(CategoryMissing, red("Unknown Category: #{@category}"))
      end

      def category_exists?
        return true if File.directory?(@category_path)
        if Gill.config.categories.has_key?(@category.to_sym)
          return true if File.directory?(cached_repo_path)
        end
        raise(CategoryMissing, red("Unknown Category: #{@category}"))
      end

      def repo_exists?
        return true if File.directory?(@repo_path) || File.directory?(cached_repo_path)
        raise(RepositoryMissing, red("Unknown Repository: #{@category}"))
      end

      def known_category?(key)
        return true if Gill.config.repos.has_key?(key.to_s.to_sym)
        false
      end

      def find_parents_for_repo(path)
        unless basename(path).to_s == basename(Gill.config.source_path).to_s
          @parent_folders << path if known_category?(basename(path))
          find_parents_for_repo(parent(path))
        end
      end

      def remove_repository_files
        find_parents_for_repo(cached_repo_path)
        remove_question =<<-QUESTION
          Are you sure you want to remove the repository: #{@repo_name} => #{@repo_path}?
        QUESTION
        ask remove_question.strip, :default => false do
          STDOUT.puts green("#{@repo_name.capitalize} removed successfully.")
          FileUtils.rm_rf(cached_repo_path)
        end

        @parent_folders.each do |path|
          if folder_empty?(path)
            unless basename(path).to_s == basename(Gill.config.default_path).to_s
              ask "Would you like to remove the empty parent folder: #{basename(path)}?", :default => false do
                STDOUT.puts green("#{path.to_s.capitalize} removed successfully.")
                FileUtils.rm_rf(path)
              end
            end
          end
        end
      end

  end
end
